/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "extra.h"
#include "bits.h"

/* expands the one keyword to three (for tripleDES)
 */
int BreakToThree(void * key, unsigned int keylen, void * keyword1, void * keyword2, void * keyword3)
{
unsigned int D,y,i,j,z;
unsigned char key_buf[5][32];  /* up to sixteen  (128/4=32) */
char *tmpkey=key;

  if (keylen<3) {
   fprintf(stderr,"Not enough long keyword for tripleDES (3 char at least)\n");
   return 1;
  }
  else {

      if (keylen>=24) keylen=24;

          D=keylen/3;
          y=keylen % 3;

          for (i=0;i<3;i++) {

              bzero(key_buf[i],sizeof(key_buf[i]));
              if (i==2) { /* last round */
                memcpy(key_buf[i],tmpkey,D+y);
              }
              else {
               memcpy(key_buf[i],tmpkey,D);    
              }
         
              for (j=0;j<D;j++) {
                for (z=0;z<(keylen-j-1);z++)
                  memcpy(&tmpkey[z],&tmpkey[z+1],1);
              }
          }

         memcpy(keyword1,key_buf[0],8); 
         memcpy(keyword2,key_buf[1],8); 
         memcpy(keyword3,key_buf[2],8); 
         return 0;
}

}


/* Byte swap a 32bit integer */
word32 byteswap(word32 x)
{
	register char *cp,tmp;

	cp = (char *)&x;
	tmp = cp[3];
	cp[3] = cp[0];
	cp[0] = tmp;

	tmp = cp[2];
	cp[2] = cp[1];
	cp[1] = tmp;

	return x;
}

char * my_getpass(char * prt) {
char *atmp;
char ztmp[128];
char *btmp;

   atmp=getpass(prt);

   bzero(ztmp,sizeof(ztmp));
   strncat(ztmp,atmp,127);

   btmp=getpass("Re-Enter key: ");
   
  if (strcmp(ztmp,btmp)!=0) {
     fprintf(stderr,"Keywords do not match.\n");
     return NULL;
  }

return btmp;
}

char * get_password(int numofchar,int bitmode,int mode,int type)
{
int x=0;
char *tmp;
char msg[100];

  if (bitmode==0) {
   sprintf(msg,"Enter key (%d chars): ",numofchar);

   if (mode==ENCRYPT) {tmp=my_getpass(msg);} else {tmp=getpass(msg);}
   if(tmp==NULL) return NULL;

   if ((strlen(tmp) > numofchar) && (type!=BLOWFISH)) { fprintf(stderr,"Warning: keyword is too small for this algorithm\n"); }
   if (strlen(tmp) > numofchar) { fprintf(stderr,"Warning: Reducing the size of keyword"); }
 }
 else { /* 7bit mode */


   if ( ((numofchar*8) % 7) !=0 ) x=1;
   sprintf(msg,"Enter key (%d chars): ",((numofchar*8)/7)+x);

   if (mode==ENCRYPT) {tmp=my_getpass(msg);} else {tmp=getpass(msg);}
   if(tmp==NULL) return NULL;

   if ((strlen(tmp) < (numofchar*8)/7) && (type!=BLOWFISH)) { fprintf(stderr,"Warning: keyword is too small for this algorithm\n"); }
   if (strlen(tmp) > ((numofchar*8)/7)+x) { fprintf(stderr,"Warning: Reducing the size of keyword\n"); }
   strip8bit(tmp,strlen(tmp));
}

return tmp;

}


int show_mode(int ende, int stype,int smode,char *output,int bitmode)
{
  
  if (stype==DES) fprintf (stderr,( "Algorithm: DES \n"));  
  if (stype==TripleDES) fprintf (stderr,( "Algorithm: 3DES\n"));
  if (stype==ThreeWAY) fprintf (stderr,( "Algorithm: 3-WAY\n"));
  if (stype==BLOWFISH) fprintf (stderr,( "Algorithm: Blowfish\n"));
  if (stype==GOST) fprintf (stderr,( "Algorithm: GOST\n"));
  if (stype==CRYPT) {
    fprintf (stderr,( "Algorithm: Unix Crypt\n"));  
  }
  else {

  if (smode==CBC) fprintf (stderr,( "Mode: CBC\n"));
  if (smode==ECB) fprintf (stderr,( "Mode: ECB\n"));
  }
  
  if (bitmode==BITS7) fprintf (stderr,( "Keyword Bit Mode: 7bits/char\n"));
  if (bitmode==BITS8) fprintf (stderr,( "Keyword Bit Mode: 8bits/char\n"));  
    
   if (ende==ENCRYPT) {
     fprintf (stderr,( "Crypting to %s ...\n"),output);
     }
   else {
     fprintf (stderr, "Decrypting to %s ...\n",output);
     }
  
return 0;  
}


#ifdef HAVE_SYS_STAT_H

int check_file(char * filename) {
struct stat ostat;

  if (stat(filename,&ostat) == 0) {
    return 1; /* exists */
  }
  else {
    return 0; /* ok */
  }


}

#endif

int my_read(int fd, void *buf, size_t count) {

  int i,x;
  char *tmpbuf=buf;
  
  for (i=0;i<count;i++) {
    x=read(fd,&tmpbuf[i],1);
    if (x==-1) return (-1);
    if (x==0) return i;
  }

return count;

}
