/*  Mini-Crypt2 program. It encrypts text or binary files using
 *  keywords. The keyword is NOT included in the encrypted file so
 *  there is no way to decrypt it, if you forgot it. 
 *  For a brief description of the algorithms read the README file and
 *  the man page.
 *                                     Nikos Mavroyanopoulos
 *                                         nmav@i-net.paiko.gr
 *
 *    Copyright (C) 1998,1999 Nikos Mavroyanopoulos
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "mcrypt.h"
#include "des.h"
#include "gost.h"
#include "3-way.h"
#include "crypt.h"
#include "blowfish.h"
#include "bits.h"
#include "lcrypt.h"
#include "sunlink.h"

int bit7_flag=BITS7;
int stream_flag=0;
int mode=CBC; /* 0:cbc 1:ecb */
int type=BLOWFISH; /* default: blowfish */
int quiet=1; /* silent by default */
int unlink_flag=0;

void usage(char *name)
{
     printf( "\n  mini-crypt encrypts and decrypts files using the des, triple des\n");
     printf( "3-way, gost and blowfish algorithms.\n\n");
     printf( "To encrypt write:\n%s (-e) FILE1 FILE2 ...\n",name);
     printf( "To decrypt write:\n%s -d FILE1 FILE2 ...\n\n",name);
     printf( "-e --encrypt      encrypts\n");
     printf( "-d --decrypt      decrypts\n");
     printf( "-t --type TYPE    specify the encryption and decryption algorithm.\n");
     printf( "   Legal parameters are: 3way,des,3des,gost,crypt and blowfish.\n");
     printf( "    (Default is blowfish)\n");
     printf( "-m --mode MODE    specify the mode (ecb or cbc). (Default is cbc)\n");
     printf( "-k --key  KEYWORD keyword to use.\n");
     printf( "-7 --7bit         keyword in 7bit mode. (default)\n");     
     printf( "-8 --8bit         keyword in 8bit mode.\n");
     printf( "-u --unlink       unlink (delete) the input file after encryption/decryption\n");
     printf( "-L --license      displays license information.\n");     
     printf( "-q --quiet        Suppresses some non critical warnings.\n");
     printf( "-V --verbose      Some more information is displayed.\n");
     printf( "-f --force        forces output to stdout.\n\n");
     printf( "-h --help         prints this help\n");
     printf( "-v --version      prints the version number\n\n");
     printf ("Report bugs to nmav@i-net.paiko.gr.\n\n");
}




/*				The main    */
int main(int argc,char **argv)
{
	short int ch,ein=0,din=0,kin=0,force=0;
	char *einfile=0, *outfile=0 ,*dinfile=0, *keyword=0,*modet=0,*typet=0;
	char **file; 
	extern char *optarg;
	int x,y,i,file_count=0;



struct option longopts[] = {
	    { "version", 0, 0, 'v' },
	    { "help", 0, 0, 'h' },
	    { "encrypt", 0, 0, 'e' },
	    { "decrypt", 0, 0, 'd' },
	    { "mode", 1, 0, 'm' },
	    { "type", 1, 0, 't' },
	    { "key", 1, 0, 'k' },
	    { "force", 0, 0, 'f' },
	    { "7bit", 0, 0, '7' },	    
	    { "8bit", 0, 0, '8' },
	    { "quiet", 0, 0, 'q'},
	    { "verbose", 0, 0, 'V'},	    
	    { "license", 0, 0, 'L'},
	    { "unlink", 0, 0, 'u'},	    
	    { 0, 0, 0, 0 }
       	  };
 
            /* File pointers are as much as the file arguments */

        file=malloc((argc-optind)*sizeof(char*)); 

	while( (ch=getopt_long(argc,argv,"f78LVuqedt:m:k:hv",longopts, (int *)0 )) != EOF )

		switch(ch) {
	
		case 'e':
			ein=1;
			if (din!=0) {fprintf (stderr, "%s: You cannot specify both encryption and decryption.\n",argv[0]);return 1;}
			break;

		case 'v':
			printf ("Mini-crypt v.%s\n",VERSION);
			printf ("Copyright (C) 1998,1999 Nikos Mavroyanopoulos (nmav@i-net.paiko.gr)\n");
			return 0;
			break;

		case 'L':
			printf ("\nCopyright (C) 1998 Nikos Mavroyanopoulos\n
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
		                    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
		                                    
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n\n");
			                                                
			return 0;
			break;			

		case 'd':
			din=1;
			if (ein!=0) {fprintf (stderr, "%s: You cannot specify both encryption and decryption.\n",argv[0]);return 1;}
			break;

		case 'f':
			force=1;
			break;			

		case 'u':
			unlink_flag=1;
			break;			

		case 'q':
			quiet=1;
			break;			

		case 'V':
			quiet=0;
			break;

		case '7':
			bit7_flag=BITS7;
			break;			

		case '8':
			bit7_flag=BITS8;
			break;			

		case 'k':
			kin=1;
			keyword= malloc(strlen(optarg)+10);
			strcpy(keyword,optarg);
			break;
		case 'm':
			modet= malloc(strlen(optarg)+1);
			strcpy(modet,optarg);
			if (modet[0]=='e' || modet[0]=='E') mode=1;
			free(modet);
			break;
		case 't':
			typet= malloc(strlen(optarg)+1);
			strcpy(typet,optarg);
			if (strncasecmp(typet,"DES",3)==0) {
			  type=DES;
			 }
			else {
			 if (strncasecmp(typet,"3DES",4)==0 || strncasecmp(typet,"3-DES",3)==0) {
                          type=TripleDES;
                         }
                         else {
                          if (strncasecmp(typet,"3WAY",4)==0 || strncasecmp(typet,"3-WAY",3)==0) {
                           type=ThreeWAY;
                          }
                          else {
                           if (strncasecmp(typet,"BLOWFISH",8)==0) {
                            type=BLOWFISH;
                           }
                           else {
                             if (strncasecmp(typet,"GOST",4)==0) {
                               type=GOST;
                             }
                             else {
                              if (strncasecmp(typet,"CRYPT",5)==0) {
                              type=CRYPT;
                              }
                             }
                           }
                          }
                         }
                        }
			free(typet);
			break;


		case '?':
		case 'h':
		default:
		     usage(argv[0]);
		     return 1;
		     break;


		}

#ifdef HAVE_SYS_STAT_H
  umask(066);
#endif

/* Normal startup */



    i=0;
    file_count+= (argc - optind);


    if (file_count==0) {
        stream_flag=1;
    }  

    while (optind < argc) {
        file[i]=argv[optind++];
        i++;
    }

    if (stream_flag==1) file_count++;


/* Do as many times as many files we got */
for(i=0;i<file_count;i++) {

    if(i!=0) free(outfile);



    if (stream_flag==0) {
      if (quiet==0) fprintf(stderr, "File: %s\n",file[i]);
    }
    else {
      if (quiet==0) fprintf(stderr, "File: stdin\n");    
    }
    
	if ( (strstr(argv[0],"decrypt")!=NULL) || (din==1)) {
            if (stream_flag!=1) dinfile=file[i];
            din=1;
             if( (isatty(fileno( (FILE *)(stdin) ) ) == 1) && (stream_flag==1) && (force==0) ) { /* not a tty */        
             fprintf(stderr,"%s: Data will not be read from a terminal. ",argv[0]);
             fprintf(stderr,"Redirect the input instead.\nUse the --help parameter for more help.\n");
             exit(1);
             }
        }
        else { /* called as mcrypt or something like that */
            if (stream_flag!=1) einfile=file[i];
            ein=1;
             if( (isatty(fileno( (FILE *)(stdout) ) ) == 1) && (stream_flag==1) && (force==0) ) { /* not a tty */        
             fprintf(stderr,"%s: Data will not be written to a terminal. ",argv[0]);
             fprintf(stderr,"Redirect the output instead.\nUse the --help parameter for more help.\n");
             exit(1);
             }            
        }


       if(stream_flag!=1) {
	 if (din==1) {

            y=strlen(dinfile);
            /* If the file finished in .enc remove it */
             
            if (dinfile[y-1]=='c' && dinfile[y-2]=='n' && dinfile[y-3]=='e' && dinfile[y-4]=='.') {

	      outfile= calloc(y-3,1);
  	      strncat(outfile,dinfile,y-4);
#ifdef HAVE_SYS_STAT_H
              /* But if it exists exit */
              if (check_file(outfile)!=0) {fprintf(stderr,"\nFile %s exists... skipping...\n\n",outfile); fflush(stderr); continue;}
#endif  	      
            }
            else { /* append .dec */

	     outfile= calloc(y+5,1);
             strncpy(outfile,dinfile,y);
             strcat(outfile,".dec");
            }
         }
	 else { /* encryption- append .enc */
	    outfile= calloc(strlen(einfile)+5,1);
	    strcpy(outfile,einfile);
	    strcat(outfile,".enc");
	 }
       }
       else { /* if streams */

         outfile=malloc(7);
         strcpy(outfile,"stdout");
       }
/* Decrypt */
	if (din==1) {

		      switch (type) {
		        case DES:
			 x=decrypt_general(DES, dinfile, outfile, keyword);
			 break;
			 
		        case TripleDES:
			 x=decrypt_general(TripleDES, dinfile, outfile, keyword);
			 break;
			 
			case ThreeWAY:
			 x=decrypt_general(ThreeWAY,dinfile,outfile,keyword);
			 break;
			case BLOWFISH:
			 x=decrypt_general(BLOWFISH,dinfile,outfile,keyword);
			 break;			 
			case GOST:
			 x=decrypt_general(GOST,dinfile,outfile,keyword);
			 break;
			case CRYPT:
			 x=encrypt_crypt(dinfile,outfile,keyword);
			 break;			 
                        default:
                         x=1;
                         break;
                        }
                        
			if (x==0) {
        	           printf ("The file was decrypted.\n\n");
#ifdef HAVE_SYS_STAT_H
        	           if (unlink_flag==1 && stream_flag==0) x=safe_unlink(dinfile);
	        	   if (x!=0) perror("safe_unlink");
#endif        	           
        	        }
	        	else {
        	           printf ("The file was NOT decrypted successfully.\n\n");
        	        }
        
	        	/* return x; */

	}		

/* Encrypt */
	if (ein==1) {
		      switch (type) {
		        case DES:
	        	 x=encrypt_general(DES,einfile,outfile,keyword);
	        	 break;
	        	 
		        case TripleDES:
	        	 x=encrypt_general(TripleDES,einfile,outfile,keyword);
	        	 break;
	        	 
	        	case ThreeWAY:
	        	 x=encrypt_general(ThreeWAY,einfile,outfile,keyword);
	        	 break;
	        	 
	        	case BLOWFISH:
	        	 x=encrypt_general(BLOWFISH,einfile,outfile,keyword);
	        	 break;	        	 
	        	 
	        	case GOST:
	        	 x=encrypt_general(GOST,einfile,outfile,keyword);
	        	 break;
	        	 
	        	case CRYPT:
	        	 x=encrypt_crypt(einfile,outfile,keyword);
	        	 break;
	        	 
	        	default:
	        	 x=1;
	        	 break;
                        }

	        	if (x==0) {
	        	   printf ("The file was successfully encrypted.\n\n");
#ifdef HAVE_SYS_STAT_H
	        	   if (unlink_flag==1 && stream_flag==0) x=safe_unlink(einfile);
	        	   if (x!=0) perror("safe_unlink");
#endif	        	   
		       	}
	                else {
	                  printf ("The file was NOT encrypted successfully.\n\n");
	                }
		}

   } /* the main loop */

return 0;

}




/* UNIX CRYPT Encryption */

int encrypt_crypt(char *fromfile,char *tofile,char *key)
{
word32 plaintext[2]; /* 64 bit */
int ch=0,how=8; /* how needs to be initialized to 8 */
char keyword[13]; 
char *tmp;
FILE *TOF;
FILE *FROMF;


/* Crypt does not care about these */
mode=2;
bit7_flag=BITS8;

/* open files */
if (stream_flag==1) {

	FROMF=stdin;
	if (FROMF==NULL) {fprintf (stderr,("Error opening stdin\n"));fclose(FROMF);return 1;}
	TOF=stdout;
	if (TOF==NULL) {fprintf (stderr,("Error opening stdout\n"));fclose(TOF);return 1;}
}
else {

	FROMF=fopen(fromfile,"r");
	if (FROMF==NULL) {perror("fopen");return 1;}
	TOF=fopen(tofile,"w");
	if (TOF==NULL) {perror("fopen");return 1;}
}

        bzero(keyword,sizeof(keyword));
	if (key==NULL) 	{
          tmp=get_password(8,bit7_flag,DECRYPT,type);
          if (tmp==NULL) return 1;

          memcpy(keyword,tmp,strlen(tmp));

	}
	else {
           if (strlen(key)>8) {key[8]='\0'; fprintf(stderr,"Warning: Reducing the size of the key\n");}
	   memcpy(keyword,key,strlen(key)); /* Copy the 64 bits in key */
	}

if (quiet==0) show_mode(ENCRYPT,type,mode,tofile,bit7_flag); 

#ifdef DEBUG
fprintf (stderr,( "\nStarted reading and writing files.\n"));
#endif


if (unix_setup(keyword)==1) {fprintf(stderr,"Cannot encrypt key\n");return 1; }/* init the crypt */

do
{
/* else read the first 1 byte of the file and stores the to plaintext */
 
                   how=my_read( fileno((FILE *)FROMF) , plaintext ,1);
                   if ( how ==-1 ) {
                      perror("read"); 
                      exit(1);
                   }
                   if (how==0) ch=1;
                   
		unix_crypt(keyword,plaintext,1,0);
                
                if (ch==0)
		 if (fwrite(plaintext,1,1,TOF)==0) {perror("fwrite"); return(1);}

}
while(ch==0);

/* close files */
        fclose(TOF);
        fclose(FROMF);

#ifdef DEBUG
fprintf (stderr,("\nFiles closed."));
#endif

return 0;
}













/* General Encryption */

int encrypt_general(int algorithm, char *fromfile,char *tofile,char *key)
{
word32 counter=0;
int i=0,j=0,ch=0,start=0,how;
char *tmp;
unsigned int len=0;
FILE *TOF;
FILE *FROMF;
int keysize=get_key_size(algorithm);
int blocksize=get_block_size(algorithm);

word32 *IV=malloc(blocksize);
word32 *keyword=calloc(keysize,1); 
word32 *ciphertext=malloc(blocksize);
word32 *ciphertextcbc=malloc(blocksize);

/* Initialisation Vector */
srand(time(0));

/* Random number generator */

for (i=0;i<blocksize/4;i++) {
 IV[i] = RAND32 ;
}


/* open files */
  if (stream_flag==1) {

	FROMF=stdin;
	if (FROMF==NULL) {fprintf (stderr,("Error opening stdin\n"));fclose(FROMF);return 1;}
	TOF=stdout;
	if (TOF==NULL) {fprintf (stderr,("Error opening stdout\n"));fclose(TOF);return 1;}
  }
  else {
	FROMF=fopen(fromfile,"r");
	if (FROMF==NULL) {perror("fopen");return 1;}
	TOF=fopen(tofile,"w");
	if (TOF==NULL) {perror("fopen");return 1;}
  }


	if (key==NULL) 	{
          tmp=get_password( keysize, bit7_flag, ENCRYPT, algorithm );
          if (tmp==NULL) return 1;
          len=strlen(tmp);

          if (len > keysize) len=keysize;
          memcpy(keyword, tmp, len);
                
	}
	else {

           if (bit7_flag==BITS8) {
               len=strlen(key);

               if (len>keysize) {
                  fprintf(stderr,"Warning: Reducing the size of the key\n");
                  len=keysize;
               }
               memcpy(keyword, key, len);
           }
           else { /* 7bit mode */
               strip8bit(key, strlen(key));
               len=strlen(key);
               if (len > keysize) {
                  fprintf(stderr,"Warning: Reducing the size of the key\n");
                  len=keysize;
               }
               memcpy(keyword, key, len);
           }           
	}

if (quiet==0) show_mode(ENCRYPT,algorithm,mode,tofile,bit7_flag);

#ifdef DEBUG
fprintf (stderr,( "\nStarted reading and writing files.\n"));
#endif

if (algorithm != BLOWFISH) len=keysize; /* Only Blowfish has non-fixed key */

init_mcrypt(algorithm,keyword,len);

do
{

/* If first time use the Init. vector */
	if (start==0 && mode==CBC) {
	 memcpy(ciphertext,IV,blocksize);
	}
	else {

/* else read the first n bytes of the file and store to ciphertext */
 
	 	 if(ch!=1) {

                   how=my_read( fileno((FILE *)FROMF) , ciphertext ,blocksize);
                   if ( how ==-1 ) {
                      perror("read"); 
                      exit(1);
                   }

  	           if (how<blocksize) {
#ifndef WORDS_BIGENDIAN
		    counter=byteswap((word32) how);
#else		    
		    counter=((word32) how);
#endif		    
                    ch=1;
                   }

                 }

	}



/* for cbc xor the plaintext with the old ciphertext block */
               if (mode==CBC && start==1) {
                   for(j=0; j<blocksize/4; j++) {
                     ciphertext[j] ^= ciphertextcbc[j];
                   }
               }
               start=1;

		mcrypt(algorithm, keyword, len, ciphertext);

/* for cbc keep the old ciphertext */
		if (mode==CBC) memcpy(ciphertextcbc,ciphertext,blocksize);

         
		if (fwrite(ciphertext,blocksize,1,TOF)==0) {perror("fwrite"); exit(1);}
		if (ch==1) {
		 if (fwrite(&counter,sizeof(counter),1,TOF)==0) {perror("fwrite"); exit(1);}
		}

}
while(ch==0);


/* close files */
        fclose(TOF);
        fclose(FROMF);

	free(IV);
	free(keyword); 
	free(ciphertext);
	free(ciphertextcbc);


#ifdef DEBUG
fprintf (stderr,("\nFiles closed."));
#endif

return 0;
}



/* General Decryption */

int decrypt_general(int algorithm, char *fromfile,char *tofile,char *key)
{
word32 counter=0;
int i=0,j=0,ch=0,start=0;
char *tmp;
int keysize=get_key_size(algorithm);
int blocksize=get_block_size(algorithm);

word32 *ciphertext_old=malloc(blocksize);
word32 *buffer=malloc(blocksize);
word32 *ciphertext=malloc(blocksize);
word32 *ciphertext_orig=malloc(blocksize);
word32 *keyword=calloc(1,keysize);

unsigned int len=0;


FILE *TOF;
FILE *FROMF;


/* open files */
if (stream_flag==1) {

	FROMF=stdin;
	if (FROMF==NULL) {fprintf (stderr,("Error opening stdin\n"));fclose(FROMF);return 1;}
	TOF=stdout;
	if (TOF==NULL) {fprintf (stderr,("Error opening stdout\n"));fclose(TOF);return 1;}
}
else {

	FROMF=fopen(fromfile,"r");
	if (FROMF==NULL) {perror("fopen");return 1;}
	TOF=fopen(tofile,"w");
	if (TOF==NULL) {perror("fopen");return 1;}
}

	if (key==NULL) {
          tmp=get_password(keysize,bit7_flag,DECRYPT,algorithm);
          if (tmp==NULL) return 1;
          len=strlen(tmp);
                              
          if (len>keysize) len=keysize;
          memcpy(keyword, tmp, len);

	}
	else {
            if (bit7_flag==BITS8) {
              len=strlen(key);
              if (len>keysize) {
                 fprintf(stderr,"Warning: Reducing the size of the key\n");
                 len=keysize;
              }
              memcpy(keyword, key, len);
            }
            else { /* 7bit mode */
               strip8bit(key,strlen(key));
               len=strlen(key);
               if (len > keysize) {
                  fprintf(stderr,"Warning: Reducing the size of the key\n");
                  len=keysize;
               }
               memcpy(keyword, key, len);
            }          
        }

if (quiet==0) show_mode(DECRYPT,algorithm,mode,tofile,bit7_flag);

#ifdef DEBUG
fprintf (stderr,( "\nStarted reading and writing files.\n"));
#endif

if (algorithm!=BLOWFISH) len=keysize;

init_mcrypt(algorithm,keyword,len);

 do
 {
  if (ch==1) break;

  /* reads the first n bytes of the file and stores the to ciphertext */
  /* /4 is here because word32 is 4 bytes */
  
        for (i=0;i<blocksize/4;i++) { 
           if(ch!=0) break;
           if ((fread(&ciphertext[i],sizeof(ciphertext[i]),1,FROMF)==0) && (feof(FROMF)==0)) {perror("fread"); exit(1);}
            
           if (feof(FROMF)!=0) {
            ch=1;
            if (i==0) {fprintf(stderr,"Incorrect file type\n"); return 1;}
#ifndef WORDS_BIGENDIAN
            counter=byteswap(ciphertext[i-1]);
#else
            counter=(ciphertext[i-1]);            
#endif            
            if (counter >= blocksize) {fprintf(stderr,"Incorrect file type\n"); return 1;}            
            clearerr(FROMF);
           }
         }

         /* Decrypt the ciphertext */

        memcpy(ciphertext_orig,ciphertext,blocksize); /* keep backup */


        mdecrypt(algorithm, keyword, len, ciphertext);
	
/* if cbc: xor the new plaintext with the old ciphertext */
        if (mode==CBC && start>0) {
          for(j=0; j<blocksize/4; j++) {
            ciphertext[j] ^= ciphertext_old[j];
          }
        }

        memcpy(ciphertext_old,ciphertext_orig,blocksize);


/* Buffer isn't immediately written here. It waits a round to do
 * so we can keep track of the end of file garbage and the IV at the begining
 */

         if (start>1 || mode==ECB) {
 
		if (ch==1) {
                  if (counter==0) break;
		  if (fwrite(buffer,counter,1,TOF)==0) {perror("fwrite"); exit(1);}
		}
		else {
		    if (mode!=1 || start!=0) {
   		      if (fwrite(buffer,blocksize,1,TOF)==0) {perror("fwrite"); exit(1);}
   		    }
                }
         }



         memcpy(buffer,ciphertext,blocksize);  /* copy the plaintext to buffer */

         start++; /* ok enter that functions */

 }
 while(ch==0);

/* close files */
        fclose(TOF);
        fclose(FROMF);

  free(ciphertext_old);
  free(buffer);
  free(ciphertext);
  free(ciphertext_orig);
  free(keyword);


#ifdef DEBUG
fprintf (stderr,("\nFiles closed."));
#endif

return 0;
}

